package com.codigoparallevar.minicards;

import android.content.Intent;
import android.support.design.widget.FloatingActionButton;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

import java.io.IOException;

public class CardActivity extends AppCompatActivity {

    public final static String INTENT = "com.codigoparallevar.minicards.CARD";
    public static final String CARD_PATH_KEY = "CARD_PATH";

    CanvasView canvasView;
    com.getbase.floatingactionbutton.AddFloatingActionButton AddPartButton;
    com.getbase.floatingactionbutton.FloatingActionButton SetDevModeButton;
    com.getbase.floatingactionbutton.FloatingActionButton SetUserModeButton;
    com.getbase.floatingactionbutton.FloatingActionButton CenterViewButton;

    com.getbase.floatingactionbutton.FloatingActionsMenu devFabMenu;
    com.getbase.floatingactionbutton.FloatingActionsMenu userFabMenu;
    com.getbase.floatingactionbutton.FloatingActionButton ShowDeckFromDevModeButton;
    com.getbase.floatingactionbutton.FloatingActionButton ShowDeckFromUserModeButton;

    boolean devMode = false;
    FloatingActionButton removePartFab;
    private PartsHolder partsHolder;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Hide action bar
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.hide();
        }

        setContentView(R.layout.activity_main);

        // Use manually controlled canvas
        canvasView = (CanvasView) findViewById(R.id.canvasView);
        canvasView.setParentActivity(this);
        String cardPath = this.getIntent().getStringExtra(CARD_PATH_KEY);
        if (cardPath == null) {
            finish();
        }

        try {
            canvasView.loadCard(cardPath);
        } catch (IOException e) {
            e.printStackTrace();
            finish();
        } catch (ErrorLoadingCardException e) {
            e.printStackTrace();
            finish();
        }

        // Initialize auxiliary elements
        partsHolder = new PartsHolder(this);

        removePartFab = (FloatingActionButton) findViewById(R.id.remove_part_fab);
        canvasView.setDropZone(
                removePartFab.getX(), removePartFab.getX() + removePartFab.getWidth(),
                removePartFab.getY(), removePartFab.getY() + removePartFab.getHeight());

        // Set callbacks to fab buttons
        devFabMenu = (com.getbase.floatingactionbutton.FloatingActionsMenu)
                findViewById(R.id.dev_mode_fab);
        userFabMenu = (com.getbase.floatingactionbutton.FloatingActionsMenu)
                findViewById(R.id.user_mode_fab);

        AddPartButton = (com.getbase.floatingactionbutton.AddFloatingActionButton)
                findViewById(R.id.add_part_button);
        AddPartButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                partsHolder.openAddPartModal(canvasView);
            }
        });

        CenterViewButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.center_view_button);
        CenterViewButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                canvasView.CenterView();
            }
        });

        SetDevModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.set_developer_mode_button);
        SetDevModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CardActivity.this.setDevMode(true);
            }
        });

        ShowDeckFromDevModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.show_deck_from_dev_mode_button);
        ShowDeckFromDevModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(DeckPreviewActivity.INTENT);
                CardActivity.this.startActivity(i);
            }
        });

        ShowDeckFromUserModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.show_deck_from_user_mode_button);
        ShowDeckFromUserModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(DeckPreviewActivity.INTENT);
                CardActivity.this.startActivity(i);
            }
        });

        SetUserModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.set_user_mode_button);
        SetUserModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CardActivity.this.setDevMode(false);
            }
        });

        setDevMode(devMode);
    }

    private void setDevMode(boolean goToDevMode) {
        boolean willInvalidate = devMode != goToDevMode;

        devMode = goToDevMode;
        if (goToDevMode){
            devFabMenu.setVisibility(View.VISIBLE);
            userFabMenu.setVisibility(View.GONE);
        }
        else {
            devFabMenu.setVisibility(View.GONE);
            userFabMenu.setVisibility(View.VISIBLE);
        }

        this.canvasView.setDevMode(devMode);

        if (willInvalidate){
            invalidate();
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (canvasView != null){
            return canvasView.onTouchEvent(event);
        }

        Log.e("CardActivity", "CanvasView is null");

        return super.onTouchEvent(event);
    }

    public void invalidate() {
        if (canvasView != null){
            if (canvasView.isDragging()){
                devFabMenu.setVisibility(View.GONE);
                userFabMenu.setVisibility(View.GONE);
                removePartFab.setVisibility(View.VISIBLE);
                Log.d("Main", "Changing visibility!");
            }
            else {
                this.setDevMode(devMode);
                removePartFab.setVisibility(View.GONE);
                Log.d("Main", "Now changing visibility!");
            }
            canvasView.setDropZone(
                    removePartFab.getX(), removePartFab.getX() + removePartFab.getWidth(),
                    removePartFab.getY(), removePartFab.getY() + removePartFab.getHeight());
        }
    }
}
